/*
** Hello World Server
*/

#include <cbcp.h>
#define CBCP_NET_TCP_IMPLEMENTATION
#include <cbcp_net_tcp.h>

#include <stdio.h>



/*
** CBCP command predeclarations
*/
static void command(CBCP_Command_Args *args);

static void command_rejected_callback(CBCP_Command_Rejected_Args args);


/*
** CBCP program
*/
int main(int argc, char const *argv[]) {
	if (argc < 2) {
		fprintf(stderr, "Usage:\n\t%s <cbcp_database_filepath>\n", argv[0]);
		return 1;
	}

	/*
	** Initialize CBCP by loading the database file for this host
	*/

	const char *cbcp_database_filepath = argv[1];
	CBCP_State *cbcp;

	cbcp_net_tcp_init(0);
	cbcp = cbcp_init(cbcp_database_filepath);

	if (cbcp == NULL) {
		fprintf(stderr, "Could not initialize CBCP.\n");
		return 1;
	}

	/*
	** Bind command callbacks
	*/

	if (!cbcp_server_init_command(
		cbcp,
		"Interface",
		"Command",
		command,
		NULL,
		0)
	) {
		fprintf(stderr, "Could not initialize served command.\n");
		return 1;
	}

	/*
	** Set rejection callback
	*/

	cbcp_server_set_command_rejected_callback(
		cbcp,
		command_rejected_callback,
		NULL);

	/*
	** Start the cbcp server
	*/

	printf("Starting CBCP server for Hello World application.\n");

	cbcp_server_start(cbcp);

	return 0;
}

static void command(CBCP_Command_Args *args) {
	(void)args;
	printf("Command executed.\n");
}

static void command_rejected_callback(CBCP_Command_Rejected_Args args) {
	fprintf(stderr, "Command rejected! Reason: %x\n", args.reason);
}

